# -*- coding: utf-8 -*-
#
# PyDitz documentation build configuration file
#
# This file is execfile()d with the current directory set to its containing
# dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os

# If extensions (or modules to document with autodoc) are in another
# directory, add these directories to sys.path here. If the directory is
# relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
for name in '.', '..':
    sys.path.insert(0, os.path.abspath(name))

# -- General configuration -----------------------------------------------------

import ditz.pkginfo as info
from ditz.version import __version__
from ditz.colorize import DitzLexer, DitzSessionLexer

# Setup function.
def setup(app):
    app.add_lexer('ditz', DitzLexer())
    app.add_lexer('ditzsession', DitzSessionLexer())

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.todo',
              'sphinx.ext.autodoc',
              'sphinx.ext.viewcode',
              'sphinx.ext.napoleon']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = info.__title__
copyright = info.__copyright__

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = __version__
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to
# some non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
unused_docs = []

# List of directories, relative to source directory, that shouldn't be
# searched for source files.
exclude_trees = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'default'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# Common substitutions.
rst_epilog = """
.. |PD|     replace:: PyDitz
.. |CMD|    replace:: ``pyditz``
.. |PDP|    replace:: ``Ditz:``
.. |RC|     replace:: ``ditz.cfg``
.. |RCDIR|  replace:: ``.ditz``
.. |RCPATH| replace:: :samp:`{HOMEDIR}/.ditz`
.. |CONF|   replace:: ``.ditz-config``

.. |GRIN|   image:: /images/face-smile.png

.. _Cmd: http://docs.python.org/2.7/library/cmd
.. _Ditz: http://rubygems.org/gems/ditz
.. _Jinja2: http://jinja.pocoo.org
.. _PEP8: http://www.python.org/dev/peps/pep-0008
.. _PyDitz: https://bitbucket.org/zondo/pyditz
.. _PyPI: https://pypi.python.org
.. _PyYAML: https://pypi.python.org/pypi/PyYAML
.. _Tkinter: https://wiki.python.org/moin/TkInter
.. _flake8: https://pypi.python.org/pypi/flake8
.. _setuptools: http://pythonhosted.org/setuptools
.. _sorttable: http://www.kryogenix.org/code/browser/sorttable
"""

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = ['.']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as
# html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the
# top of the sidebar.
#html_logo = "static/logo.png"

# The name of an image file (within the static path) to use as favicon of
# the docs.  This file should be a Windows icon file (.ico) being 16x16 or
# 32x32 pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets)
# here, relative to this directory. They are copied after the builtin
# static files, so a file named "default.css" will overwrite the builtin
# "default.css".
html_static_path = ['issues']

# If not '', a 'Last updated on:' timestamp is inserted at every page
# bottom, using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_use_modindex = True

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = True

# If true, an OpenSearch description file will be output, and all pages
# will contain a <link> tag referring to it.  The value of this option must
# be the base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = 'gindoc'


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples (source start
# file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  (master_doc, 'pyditz.tex', u'PyDitz Documentation',
   info.__author__, 'manual'),
]

# The name of an image file (relative to this directory) to place at the
# top of the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are
# parts, not chapters.
#latex_use_parts = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_use_modindex = True

# -- Options for PDF output --------------------------------------------------

# Grouping the document tree into PDF files. List of tuples (source start
# file, target name, title, author, documentclass [howto/manual]).
pdf_documents = [
  (master_doc, 'pyditz', u'PyDitz Documentation',
   info.__author__, 'manual'),
]

# -- Options for autodoc -----------------------------------------------------

autodoc_member_order = 'bysource'

autodoc_default_flags = ['members']
